# frozen_string_literal: true

module FaradayCdnMetrics
  module ServerCaches
    class Fastly
      def initialize(response_headers = {})
        @response_headers = response_headers
      end

      def hit?
        x_cache_hit?
      end

      def miss?
        x_cache_miss?
      end

      def pass?
        x_cache_miss? && fetch_age.nil?
      end

      def hit_while_revalidate?
        max_age = fetch_max_age.to_i
        age = fetch_age.to_i

        x_cache_hit? && max_age > 0 && age > max_age
      end

      private

      attr_reader :response_headers

      def x_cache_hit?
        response_headers.fetch('X-Cache', nil) == 'HIT'
      end

      def x_cache_miss?
        response_headers.fetch('X-Cache', nil) == 'MISS'
      end

      def fetch_max_age
        response_headers.fetch('Cache-Control', '').match(/max-age=(\d+)/)&.captures&.first
      end

      def fetch_age
        response_headers.fetch('Age', nil)
      end
    end
  end
end
